#########################################
# Qmethod analysis logistics policy lab #
#########################################

# Clear R environment and load libraries
rm(list=ls())
library(tidyverse)
library(qmethod) #see https://github.com/aiorazabala/qmethod
library(ggpubr)

# Load anonymised q-sorts and select columns
data <- read_csv2("qmethod/source/qsorts_anonymous.csv") # downloaded from qsortware.net, openened in excel, saved as csv with headers
data[,-(1:6)] -> q
as.data.frame(q) -> q
row.names(q) <- 1:nrow(q)
qsort <- t(q)

# Run Q-method function and check statistical outcomes
result <- qmethod(qsort, 
            nfactors = 2,
            extraction = "PCA",
            rotation = "varimax",
            forced = TRUE,
            cor.method = "pearson",
            )

print(result) #overall outcome
print(result$qdc) #distinguishing power of statements


# Diagram of descending distinguishing statements
dis <- result$qdc |> rownames_to_column() 

## Standard qmethod package plot (not used in publication)
# plot(result, xlab = 'z-scores', ylab = 'statements',
#      pchlist = NULL, colours = NULL,
#      fnames = NULL, legend = TRUE,
#      dist = TRUE, pchlist.fill = NULL,
#      leg.pos="bottomright", xlim= NULL,
#      sort.items=T, factors = NULL)

## GGplot diagram, including statement text and custom axes
statements <- read_csv2("qmethod/source/statements.csv") #load qsort statement text

### Format and join data
z_scores <- as_tibble(result$zsc) |> rownames_to_column() |> 
  mutate(rowname = paste("s",rowname, sep = ""))
plotdata <- statements |> left_join(dis) |> left_join(z_scores) |> 
  arrange(abs(f1_f2)) |> mutate(rowname = factor(rowname, levels = rowname)) |>
                                        pivot_longer(cols = c(zsc_f1,zsc_f2))

### Plot and save diagram
ggplot(plotdata, aes(y = rowname, x = value, color = name)) +
  geom_vline(xintercept = 0, color = "gray80", size = 0.2) +
  geom_point(size = 6, alpha = 1) +
  geom_text(data = filter(plotdata, name=="zsc_f1"),
            aes(label = EN, x = mean(range(value))),
            size = 3, color = "gray60") +
  theme_bw(base_size = 12) +
  theme(legend.position="bottom",
        #panel.border = element_blank(), 
        panel.grid.major = element_blank(),
        panel.grid.minor = element_blank(),
        #axis.line = element_line(colour = "black")
        ) + 
  labs(title = "", 
       x = expression("disagree" %<->% "agree"), 
       y = expression("consensus" %<->% "distinguishing")) +
  scale_color_manual(name = "factor", values = c("red","blue"), labels = c("View 1","View 2"))
ggsave(filename = "qmethod/plot/qmethod_plot.pdf",
       dpi = 1200, width = 200, height = 200, units = "mm")
ggsave(filename = "qmethod/plot/qmethod_plot.png",
       dpi = 300, width = 3000, height = 3000, units = "px")


# Plot validity and applicability diagrams

## Join individual data to group outcome
flags <- as_tibble(result$flagged)
flags$id <- data$id
groupdata <- data |> select((1:7),s15,s16,s18,s24) |> left_join(flags) |> mutate(view = case_when(flag_f1 == TRUE ~ 1,
                                                                                                  flag_f2 == TRUE ~ 2))
plotdata <- groupdata |> drop_na()
plotdata$view <- as.factor(plotdata$view)

## calculate group means for two most relevant validity and applicability statements
gm1 <- plotdata |>
  group_by(view) |>
  summarise(
    VAR1 = mean(s15),
    VAR2 = mean(s24)
  )
gm2 <- plotdata |>
  group_by(view) |>
  summarise(
    VAR1 = mean(s16),
    VAR2 = mean(s18)
  )

## Plots
a <- ggplot(plotdata, aes(x = s15, y = s24, col = as.factor(view))) +
  geom_jitter() +
  geom_point(data = gm1, aes(x = VAR1, y = VAR2, col = as.factor(view)),size = 5) +
  scale_color_manual(name = "factor", values = c("red","blue"), labels = c("View 1","View 2")) +
  theme_bw(base_size = 8) + theme(legend.position = "bottom") +
  labs(title = "validity",
       x = "nature and landscape weighed too high in the map (s15)",
       y = "the map is subjective (s24)")

b <- ggplot(plotdata, aes(x = s16, y = s18, color = as.factor(view))) +
  geom_jitter() +
  geom_point(data = gm2, aes(x = VAR1, y = VAR2, col = as.factor(view)),size = 5) +
  scale_color_manual(name = "factor", values = c("red","blue"), labels = c("View 1","View 2")) +
  theme_bw(base_size = 8) + theme(legend.position = "bottom") +
  labs(title = "applicability",
       x = "the map is a useful planning tool (s16)",
       y = "the map helps the planning of large new cluster (s18)")

## Save plot combination
ggarrange(a,b, ncol = 2, nrow = 1)
ggsave(filename = "qmethod/plot/validity_applicability.pdf",
       dpi = 1200, width = 200, height = 100, units = "mm")
ggsave(filename = "qmethod/plot/validity_applicability.png",
       dpi = 300, width = 1500, height = 1500, units = "px")